import os

import click
from ruamel.yaml import safe_load, safe_dump


class Configuration(object):
    """read write conf yaml files"""

    def __init__(self, conf_file=None):
        self.conf_file = (
            conf_file if conf_file else os.path.join(os.path.dirname(__file__), "conf.yaml")
        )

    def read(self):
        with open(self.conf_file, "r") as ymlfile:
            return safe_load(ymlfile)

    def write(self, cfg):
        with open(self.conf_file, "w") as ymlfile:
            return safe_dump(cfg, ymlfile, default_flow_style=False)


@click.command(help="Configure Miq Selenium Server")
def config():
    """Config command"""

    conf = Configuration()
    cfg = conf.read()

    cfg["container"]["project"] = click.prompt(
        "Miq project working directory", default=cfg["container"]["project"]
    )
    cfg["container"]["name"] = click.prompt("Container name", default=cfg["container"]["name"])
    cfg["container"]["image"] = click.prompt(
        "Docker selenium driver image", default=cfg["container"]["image"]
    )
    cfg["container"]["vnc_port"] = click.prompt(
        "VNC running on port?", default=cfg["container"]["vnc_port"]
    )
    cfg["container"]["server_port"] = click.prompt(
        "Selenium server running on port?", default=cfg["container"]["server_port"]
    )
    cfg["container"]["data_dir"] = click.prompt(
        "Testing data mount to directory '/var/tmp'", default=cfg["container"]["data_dir"]
    )
    conf.write(cfg=cfg)
    click.echo("Configuration saved successfully...")



import time
from subprocess import PIPE, Popen

import click
import docker

from miqsel.config import Configuration
from miqsel.env import LocalEnv


class Server(object):
    """Selenium Server Management"""

    def __init__(self):
        self.client = docker.from_env(version="auto")
        self.cfg = Configuration().read()

    @property
    def container(self):
        """
        Get running selenium container
        :return: if container running return container object else None
        """

        try:
            return self.client.containers.get(self.cfg["container"]["name"])
        except docker.errors.NotFound:
            return None

    @property
    def hostname(self):
        """
        Get ip allocated to container
        :return: If container running return ip else None
        """

        return self.container.attrs["NetworkSettings"]["IPAddress"] if self.container else None

    @property
    def executor(self):
        """
        Get executor url
        :return: If container running return executor url else None
        """

        return (
            "http://{host}:{port}/wd/hub".format(
                host=self.hostname, port=self.cfg["container"]["server_port"]
            )
            if self.hostname
            else None
        )

    @property
    def vnc(self):
        """
        Get vnc url
        :return: If container running return vnc url else None
        """

        return (
            "{host}:{port}".format(host=self.hostname, port=self.cfg["container"]["vnc_port"])
            if self.hostname
            else None
        )

    def start(self, **kwargs):
        """Start selenium container"""

        img = self.cfg["container"]["image"]
        name = self.cfg["container"]["name"]
        mount_dir = self.cfg["container"]["data_dir"]

        if mount_dir != "None":
            kwargs.update({"volumes": {mount_dir: {"bind": "/var/tmp", "mode": "ro"}}})

        if not self.container:
            if not self.client.images.list(name=img):
                click.echo("Pulling docker images...")
                click.echo("It will take some time; Please wait...")

            self.client.containers.run(img, name=name, detach=True, auto_remove=True, **kwargs)

            time.sleep(5)

            t0 = time.time()
            while True:
                if self.hostname:
                    break
                elif time.time() > (t0 + 30):
                    click.echo("Timeout: Fail to get hostname. Check for selenium server status")
                    exit(0)

        elif getattr(self.container, "status", None) == "exited":
            self.container.start()
        else:
            click.echo("Need to check container status...")

    def stop(self):
        """Stop selenium container if running"""

        if getattr(self.container, "status", None) == "running":
            self.container.stop()

    @property
    def status(self):
        """
        Get status of selenium container
        :return: container status if running else stopped
        """

        if self.container:
            return self.container.status
        else:
            return "stopped"


@click.command(help="Status of Selenium Server")
def status():
    """status command"""

    miq = Server()
    click.echo(miq.status)


@click.command(help="Start Selenium Server")
@click.pass_context
def start(ctx):
    """start command"""

    miq = Server()
    if miq.status == "stopped":
        try:
            miq.start()
            click.echo("Selenium Server started")
        except Exception:
            click.echo("Fail to start Selenium Server")
            raise

        env = LocalEnv()
        env.executor = miq.executor
        ctx.invoke(viewer)
    else:
        click.echo("Server in running state")


@click.command(help="Stop Selenium Server")
def stop():
    """stop command"""

    miq = Server()
    if miq.status == "running":
        miq.stop()
        click.echo("Selenium Server stopped")
    else:
        click.echo("Server not running...")


@click.command(help="Selenium executor URL")
def executor():
    """executor url command"""

    miq = Server()
    if miq.status == "running":
        click.echo(miq.executor)
    else:
        click.echo("Server not running...")


@click.command(help="VNC URL")
def vnc():
    """vnc url command"""

    miq = Server()
    if miq.status == "running":
        click.echo(miq.vnc)
    else:
        click.echo("Server not running...")


@click.command(help="VNC viewer")
@click.option("-u", "--url", default=None, help="Server url with port <hostname:port>")
def viewer(url):
    """vnc viewer command

    :param url: Server url with port <hostname:port>
    """

    url = url if url else Server().vnc
    if url:
        try:
            Popen(["vncviewer", url], stdout=PIPE)
        except FileNotFoundError:
            click.echo("Need vnc viewer... Check README")
    else:
        click.echo("Server not running...")



import os

import click
from ruamel.yaml import safe_load, safe_dump

from miqsel.config import Configuration


class LocalEnv(object):
    """Local env file management"""

    def __init__(self):
        self.cfg = Configuration().read()

    @property
    def project(self):
        """ Project directory

        :return: project directory
        """
        return self.cfg["container"]["project"]

    @property
    def env_file(self):
        """ Environment file

        :return: environment file if not None
        """
        if os.path.isdir("conf"):
            # First check miqsel running from project dir
            return os.path.join("conf", "env.local.yaml")
        elif self.project:
            # Second check miqsel project path set or not
            return os.path.join(self.project, "conf", "env.local.yaml")
        else:
            return os.path.join("tmp", "env.local.yaml")

    @property
    def in_env(self):
        """ Check we are in env or not.
        :return: return bool
        """
        proj_dir = os.path.dirname(self.env_file)
        return os.path.isdir(proj_dir)

    def read(self):
        """Read Environment file

        :return: dict
        """
        with open(self.env_file, "r") as ymlfile:
            return safe_load(ymlfile)

    def write(self, cfg):
        """ Write Environment file

        :param cfg: dict to write
        """
        if self.in_env:
            with open(self.env_file, "w") as ymlfile:
                return safe_dump(cfg, ymlfile, default_flow_style=False)
        else:
            click.echo(
                "Project directory need set or run from Project directory; 'env.yaml' not updated"
            )

    @property
    def env(self):
        """Read environment
        :return: if env file exist return else sample conf
        """
        return self.read() if os.path.isfile(self.env_file) else self.cfg["miq"]

    @property
    def appliance(self):
        """appliance from env

        :return: return current appliance
        """
        return self.env["appliances"][0]["hostname"]

    @appliance.setter
    def appliance(self, value):
        """Setter for appliance

        :param value: appliance
        """
        if self.appliance != value:
            cfg = self.env
            cfg["appliances"][0]["hostname"] = value
            self.write(cfg)

    @property
    def executor(self):
        """Executor form env

        :return: executor url
        """
        return self.env["browser"]["webdriver_options"]["command_executor"]

    @executor.setter
    def executor(self, value):
        """Setter for executor

        :param value: executor url
        """
        if self.browser != value:
            cfg = self.env
            cfg["browser"]["webdriver_options"]["command_executor"] = value
            self.write(cfg)

    @property
    def browser(self):
        """Browser from env"""
        return self.env["browser"]["webdriver_options"]["desired_capabilities"]["browserName"]

    @browser.setter
    def browser(self, value):
        """Setter for browser

        :param value: browser (chrome/ firefox)
        """
        if self.browser != value:
            cfg = self.env
            cfg["browser"]["webdriver_options"]["desired_capabilities"]["browserName"] = value
            self.write(cfg)


@click.command(help="Set Browser")
@click.option("-c", "--chrome", "browser", flag_value="chrome", help="Chrome")
@click.option("-f", "--firefox", "browser", flag_value="firefox", help="Firefox")
def browser(browser):
    """Browser command

    :param browser: --chrome[-c] or --firefox[-f]
    """

    env = LocalEnv()

    if browser:
        click.echo("Browser set to {}".format(browser))
        env.browser = browser
    else:
        click.echo(env.browser)


@click.command(help="Set Appliance")
@click.option("-s", "--set", "app", default=None, help="Set Appliance")
def appliance(app):
    """Appliance command

    :param app: --set[-s]
    """

    env = LocalEnv()

    if app:
        click.echo("Appliance set to {}".format(app))
        env.appliance = app
    else:
        click.echo(env.appliance)



import pytest

from tests import miqsel_cmd

DATA = {
    "start": {"status": "running", "msg": "Selenium Server started"},
    "stop": {"status": "stopped", "msg": "Selenium Server stopped"},
}


def test_miqsel_help():
    """Check Status of miqsel"""

    out, _, returncode = miqsel_cmd("miqsel")
    assert returncode == 0
    assert "Usage: miqsel [OPTIONS] COMMAND [ARGS]" in out


@pytest.mark.parametrize("state", DATA)
def test_miqsel_status(state, ensure_stopped):
    """Check Status of unconfigured miqsel"""

    cmd = "miqsel {}".format(state)
    miqsel_cmd(cmd)
    out, _, returncode = miqsel_cmd("miqsel status")
    assert returncode == 0
    assert out.strip() == DATA[state]["status"]


@pytest.mark.parametrize("state", DATA)
def test_miqsel(state, ensure_stopped):
    """Check start and stop of unconfigured miqsel"""
    cmd = "miqsel {}".format(state)
    out, _, returncode = miqsel_cmd(cmd)
    assert returncode == 0

    assert DATA[state]["msg"] in out

    if state == "start" and DATA[state]["status"] == "running":
        # For unconfigured miqsel evm.yaml will not update
        assert "'env.yaml' not updated" in out

    out, _, returncode = miqsel_cmd("miqsel status")
    assert returncode == 0
    assert out.strip() == DATA[state]["status"]


def test_appliance_browser(ensure_running):
    """Check appliance and browser"""
    out, _, returncode = miqsel_cmd("miqsel browser")
    assert returncode == 0
    assert out.strip() == "chrome"

    out, _, returncode = miqsel_cmd("miqsel appliance")
    assert returncode == 0
    assert out.strip() == ""



import os

import pytest

from tests import miqsel_cmd

DATA = {
    "start": {"status": "running", "msg": "Selenium Server started"},
    "stop": {"status": "stopped", "msg": "Selenium Server stopped"},
}


@pytest.mark.parametrize("state", DATA)
def test_miqsel(state, proj_dir, ensure_stopped):
    """Check start and stop of unconfigured miqsel but run from project"""

    cmd = "miqsel {}".format(state)
    out, _, returncode = miqsel_cmd(cmd)
    assert returncode == 0
    assert DATA[state]["msg"] in out

    if state == "start":
        # For unconfigured miqsel evm.local.yaml will not update
        env_path = os.path.join(proj_dir, "env.local.yaml")
        assert "'env.yaml' not updated" not in out

        # env.local.yaml will create in conf
        assert os.path.isfile(path=env_path)

        with open(env_path, "r") as f:
            assert "command_executor: http://172" in f.read()

    out, _, returncode = miqsel_cmd("miqsel status")
    assert returncode == 0
    assert out.strip() == DATA[state]["status"]


def test_appliance(proj_dir, ensure_stopped):
    """Check appliance set"""
    miqsel_cmd("miqsel start")

    # without appliance set
    out, _, returncode = miqsel_cmd("miqsel appliance")
    assert returncode == 0
    assert out.strip() == ""

    # with appliance set
    app = "192.168.1.1"
    out, _, returncode = miqsel_cmd("miqsel appliance -s {}".format(app))
    assert returncode == 0
    assert out.strip() == "Appliance set to {}".format(app)
    out, _, returncode = miqsel_cmd("miqsel appliance")
    assert out.strip() == app


def test_browser(proj_dir, ensure_stopped):
    """Check browser set"""
    miqsel_cmd("miqsel start")

    # without browser set
    out, _, returncode = miqsel_cmd("miqsel browser")
    assert returncode == 0
    assert out.strip() == "chrome"

    # with browser set
    out, _, returncode = miqsel_cmd("miqsel browser -f")
    assert returncode == 0
    assert out.strip() == "Browser set to firefox"
    out, _, returncode = miqsel_cmd("miqsel browser")
    assert out.strip() == "firefox"


def test_executor_url(proj_dir, ensure_stopped):
    """test executor pointing to"""

    miqsel_cmd("miqsel start")
    out, _, returncode = miqsel_cmd("miqsel executor")
    assert returncode == 0
    assert out.strip() == "http://172.17.0.2:4444/wd/hub"


def test_vnc_url(proj_dir, ensure_stopped):
    """test vnc pointing to"""

    miqsel_cmd("miqsel start")
    out, _, returncode = miqsel_cmd("miqsel vnc")
    assert returncode == 0
    assert out.strip() == "172.17.0.2:5999"



import sys

from setuptools import find_packages, setup

assert sys.version_info >= (3, 6, 0), "miqsel requires Python 3.6+"


with open("README.md") as readme_file:
    readme = readme_file.read()

install_requirements = ["Click>=5.0", "docker>=3.1", "ruamel.yaml~=0.15"]

setup_requirements = ["setuptools_scm"]

setup(
    author="Nikhil Dhandre",
    author_email="nik.digitronik@live.com",
    classifiers=[
        "Natural Language :: English",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Software Development :: Quality Assurance",
        "Topic :: Software Development :: Testing",
    ],
    description="Miq Selenium Server for local testing",
    entry_points={"console_scripts": ["miqsel=miqsel:main"]},
    install_requires=install_requirements,
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    setup_requires=setup_requirements,
    python_requires=">=3.6",
    use_scm_version=True,
    keywords="miqsel",
    name="miqsel",
    packages=find_packages(include=["miqsel"]),
    url="https://github.com/digitronik/miqsel",
    license="GPLv3",
    zip_safe=False,
)



import click

from miqsel.config import config
from miqsel.env import appliance, browser
from miqsel.server import executor, start, status, stop, viewer, vnc


@click.version_option()
@click.group()
def main():
    """Miq Selenium Server"""
    pass


# config command
main.add_command(config)

# server management command
main.add_command(status)
main.add_command(start)
main.add_command(stop)
main.add_command(viewer)
main.add_command(vnc)
main.add_command(executor)

# Env management command
main.add_command(appliance)
main.add_command(browser)

if __name__ == "__main__":
    main()
